import { ListItem } from '@limetech/lime-elements';
import {
    LimeProperty,
    LimeWebComponent,
    LimeWebComponentContext,
    LimeWebComponentPlatform,
    Operator,
    PlatformServiceName,
    Query,
    QueryResponse,
} from '@limetech/lime-web-components';
import { Component, Event, EventEmitter, h, Prop } from '@stencil/core';
import {
    ArticleSelectorComponent,
    CpqArticle,
    CpqArticleGroup,
    CpqItem,
} from 'limepkg-cpq-order-lwc-components';

// Extend the standard CPQArticle with a custom "stock" property
interface ArticleWithStock extends CpqArticle {
    stock: number;
}

// Extend the standard CPQItem to accept the article with stock instead of a standard article
interface Item extends CpqItem {
    article: ArticleWithStock;
}

@Component({
    tag: 'example-article-selector-stock',
    shadow: true,
})
export class ArticleSelectorStock
    implements LimeWebComponent, ArticleSelectorComponent
{
    @Prop()
    public platform: LimeWebComponentPlatform;

    @Prop()
    public context: LimeWebComponentContext;

    @Prop()
    public articleGroupProperty: LimeProperty;

    @Prop()
    public articleProperty: LimeProperty;

    @Prop()
    public item: Item;

    @Event()
    itemChange: EventEmitter<Item>;

    private articles: ArticleWithStock[] = [];

    public render() {
        return (
            <coc-article-selector
                platform={this.platform}
                context={this.context}
                articleGroupProperty={this.articleGroupProperty}
                item={this.item}
            >
                <limel-picker
                    slot="article"
                    label={this.articleProperty.localname}
                    onChange={this.handleChange}
                    searcher={this.searchArticle}
                    value={this.mapArticleToListItem(this.item.article)}
                    required={true}
                />
            </coc-article-selector>
        );
    }

    private searchArticle = async (query: string): Promise<ListItem[]> => {
        this.articles = await getArticles(
            this.platform,
            this.item.articleGroup,
            query
        );

        return this.articles.map(article => this.mapArticleToListItem(article));
    };

    private mapArticleToListItem(article: ArticleWithStock): ListItem {
        if (!article) {
            return;
        }

        return {
            text: article.description,
            secondaryText: `${article.stock} in stock`,
            value: article.id,
        };
    }

    private handleChange = (event: CustomEvent<any>) => {
        event.stopPropagation();

        let article: ArticleWithStock | undefined;
        if (event.detail) {
            article = this.articles?.find(
                item => item.id === event.detail.value
            );
        }

        const item: Item = {
            ...this.item,
            article: article || null,
            description: article?.description,
            pricePerUnit: article?.price,
            vat: article?.vat,
            costPrice: article?.costPrice,
            unit: article?.unit,
        };

        // Lower the quantity if there's not enough in stock
        if (article && article.stock < item.quantity) {
            item.quantity = article.stock;
        }

        this.itemChange.emit(item);
    };
}

// Based on services/article.ts, modified with hardcoded properties and extra stock property
async function getArticles(
    platform: LimeWebComponentPlatform,
    articleGroup?: CpqArticleGroup,
    searchText?: string
): Promise<ArticleWithStock[]> {
    const orderBy = {
        description: 'ASC',
    };

    const filter: any = {
        op: Operator.AND,
        exp: [
            // Article must always be active
            {
                op: Operator.EQUALS,
                key: 'active',
                exp: true,
            },
            // If the article belongs to an articleGroup, the articleGroup must also be active
            {
                op: Operator.OR,
                exp: [
                    // Has an articleGroup and it's active
                    {
                        op: Operator.EQUALS,
                        key: 'articlegroup.active',
                        exp: true,
                    },
                    // Has no articleGroup
                    {
                        op: Operator.EQUALS,
                        key: 'articlegroup',
                        exp: null,
                    },
                ],
            },
        ],
    };

    if (articleGroup) {
        filter.exp.push({
            op: Operator.EQUALS,
            key: 'articlegroup',
            exp: articleGroup.id,
        });
    }

    for (const word of searchText.split(' ')) {
        filter.exp.push({
            op: Operator.OR,
            exp: [
                {
                    op: Operator.LIKE,
                    key: 'description',
                    exp: word,
                },
                {
                    op: Operator.LIKE,
                    key: 'articleno',
                    exp: word,
                },
            ],
        });
    }

    const query: Query = {
        limetype: 'article',
        limit: 100,
        orderBy: [orderBy],
        filter: filter,
        responseFormat: {
            object: {
                _id: null,
                stock: null,
                articleno: null,
                description: null,
                unit: null,
                price: null,
                costprice: null,
                grossmarginratio: null,
                vat: null,
                articlegroup: {
                    _id: null,
                    name: null,
                },
            },
        },
    };

    const queryService = platform.get(PlatformServiceName.Query);

    const result: QueryResponse = await queryService.execute(query);

    return result.objects.map(mapArticleResult);
}

// Based on services/article.ts, modified with hardcoded properties and extra stock property
function mapArticleResult(result: Record<string, any>): ArticleWithStock {
    if (!result) {
        return;
    }

    const {
        _id: id,
        stock,
        articleno,
        description,
        unit,
        price,
        costprice,
        grossmarginratio,
        vat,
        articlegroup: { _id: articleGroupId, name: articleGroupName },
    } = result;

    return {
        id: id,
        stock: stock,
        articleno: articleno,
        description: description,
        unit: unit,
        price: price,
        costPrice: costprice,
        grossMarginRatio: grossmarginratio,
        vat: vat,
        articleGroup: {
            id: articleGroupId,
            name: articleGroupName,
        },
    };
}
